@@ -26,7 +26,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <sys/stat.h>
-#include <curl/curl.h>
+#include <gnurl/curl.h>
 
 /*
  * This example shows a HTTP PUT operation. PUTs a file given as a command
