@@ -206,7 +206,7 @@
 
 /* ================================================================ */
 /* No system header file shall be included in this file before this */
-/* point. The only allowed ones are those included from curl/system.h */
+/* point. The only allowed ones are those included from gnurl/system.h */
 /* ================================================================ */
 
 /*
@@ -335,7 +335,7 @@
 #endif
 
 #ifndef STDC_HEADERS /* no standard C headers! */
-#include <curl/stdcheaders.h>
+#include <gnurl/stdcheaders.h>
 #endif
 
 #ifdef __POCC__
